/*----------------------------------------------------------------------------
Program     : utl/qry/qry-mgr-compound.i

Description : Interface for character, decimal, and integer compound &
              compound range searches

Copyright(c): Taylor Steel Inc. 2004 $n
Author      : Tim Kuehn
Created     : 3/2/2005 TAYLOR STEEL INC
Notes       :

PARAMETERS  :
--------Revision History---------------------------------------------------

Last change: TDK 21/03/2005 11:47:15 AM
----------------------------------------------------------------------------*/

/******************************************************************************/
PROCEDURE add-qrymgr-query-buffer-compound-search-spec-{&data-type}-range   {&super-header}:
/******************************************************************************
PURPOSE:

    Add an date / time range to a query buffer spec.

PARAMETERS:

    cur-query-handle                    Handle to the query

    cur-buffer-num                      Buffer position in the query

    cur-primary-field-name              Name of the date field to range match against
    cur-secondary-field-name            Name of the time field to range match against

    cur-primary-start                   Starting primary range.     Ignored if this is ? or {&ignore-value}
    cur-secondary-start                 Starting secondary range.   Ignored if this is ? or {&ignore-value}

    cur-primary-end                     Ending primary range.       Ignored if this is ? or {&ignore-value}
    cur-secondary-end                   Ending secondary range.     Ignored if this is ? or {&ignore-value}

    cur-flag-list                       Optional flags to control behavior.

                                        Flag                    Description
                                        ---------------         ---------------------------------
                                        "mandatory"             Don't ignore ? date or time parameter
                                                                values

                                        "mandatory-primary"     Don't ignore ? or {&ignore-value} parameter values
                                                                Secondary time values of or {&ignore-value} may
                                                                be ignored if mandatory-secondary flag is not set

                                        "mandatory-secondary"   Don't ignore or {&ignore-value} parameter values
                                                                Has no effect if the value is or {&ignore-value}
                                                                and mandatory / mandatory-primary flag is not set
    NOTE:

        If cur-start-primary    = cur-end-primary   AND
           cur-start-secondary  = cur-end-secondary THEN
           assume this is an equality match
                                                
	Last change: TSI 3/2/2005 12:05:33 PM
******************************************************************************/
/* Local Parameters */

DEFINE INPUT PARAMETER cur-query-handle             AS HANDLE       NO-UNDO.
DEFINE INPUT PARAMETER cur-buffer-num               AS INTEGER      NO-UNDO.

DEFINE INPUT PARAMETER cur-primary-field-name       AS CHARACTER    NO-UNDO.
DEFINE INPUT PARAMETER cur-secondary-field-name     AS CHARACTER    NO-UNDO.

DEFINE INPUT PARAMETER cur-primary-start            AS {&data-type} NO-UNDO.
DEFINE INPUT PARAMETER cur-secondary-start          AS {&data-type} NO-UNDO.

DEFINE INPUT PARAMETER cur-primary-end              AS {&data-type} NO-UNDO.
DEFINE INPUT PARAMETER cur-secondary-end            AS {&data-type} NO-UNDO.

DEFINE INPUT PARAMETER cur-flag-list                AS CHARACTER    NO-UNDO.

/****************************************************************************/
&IF (DEFINED (super-header) = 0) &THEN
/****************************************************************************/

        /* This is really an equality match     */

IF cur-primary-start    = cur-primary-end       AND
   cur-secondary-start  = cur-secondary-end     THEN

    DO:

    RUN add-qrymgr-query-buffer-compound-search-spec-{&data-type}(  cur-query-handle,
                                                                    cur-buffer-num,

                                                                    cur-primary-field-name,     cur-secondary-field-name,
                                                                    "=",
                                                                    cur-primary-start,          cur-secondary-start,

                                                                    cur-flag-list).

    END.

ELSE    /* This is a range lookup               */

    DO:

    RUN add-qrymgr-query-buffer-compound-search-spec-{&data-type}(  cur-query-handle,
                                                                    cur-buffer-num,

                                                                    cur-primary-field-name,     cur-secondary-field-name,
                                                                    ">=",
                                                                    cur-primary-start,          cur-secondary-start,

                                                                    cur-flag-list).


    RUN add-qrymgr-query-buffer-compound-search-spec-{&data-type}(  cur-query-handle,
                                                                    cur-buffer-num,

                                                                    cur-primary-field-name,     cur-secondary-field-name,
                                                                    "<=",
                                                                    cur-primary-end,            cur-secondary-end,

                                                                    cur-flag-list).
    END.


&ENDIF
END PROCEDURE.

/******************************************************************************/
PROCEDURE add-qrymgr-query-buffer-compound-search-spec-{&data-type}    {&super-header}:
/******************************************************************************
PURPOSE:


PARAMETERS:

    cur-query-handle            Handle to the query

    cur-buffer-num              Buffer position in the query

    cur-primary-field-name      Name of the primary field in the buffer
    cur-secondary-field-name    Name of the secondary field in the buffer

    cur-boolean-op              One of <>, =, >=, <=, >, <

    cur-primary-value           Value of the primary field for (range) matching
    cur-secondary-value         Value of the secondary field for (range) matching

    cur-flag-list               Optional flags to control behavior.

                                Flag                    Description
                                ---------------         ---------------------------------
                                "mandatory"             Don't ignore ? date or time parameter
                                                        values

                                "mandatory-primary"     Don't ignore ? or {&ignore-value} parameter values
                                                        Secondary time values of or {&ignore-value} may
                                                        be ignored if mandatory-secondary flag is not set

                                "mandatory-secondary"   Don't ignore or {&ignore-value} parameter values
                                                        Has no effect if the value is or {&ignore-value}
                                                        and mandatory / mandatory-primary flag is not set


	Last change: TSI 3/2/2005 10:56:18 AM
******************************************************************************/
/* Local Parameters */

DEFINE INPUT PARAMETER cur-query-handle             AS HANDLE       NO-UNDO.
DEFINE INPUT PARAMETER cur-buffer-num               AS INTEGER      NO-UNDO.

DEFINE INPUT PARAMETER cur-primary-field-name       AS CHARACTER    NO-UNDO.
DEFINE INPUT PARAMETER cur-secondary-field-name     AS CHARACTER    NO-UNDO.

DEFINE INPUT PARAMETER cur-boolean-op               AS CHARACTER    NO-UNDO.

DEFINE INPUT PARAMETER cur-primary-value            AS {&data-type} NO-UNDO.
DEFINE INPUT PARAMETER cur-secondary-value          AS {&data-type} NO-UNDO.

DEFINE INPUT PARAMETER cur-flag-list                AS CHARACTER    NO-UNDO.

/****************************************************************************/
&IF (DEFINED (super-header) = 0) &THEN
/****************************************************************************/
/* Local Variables */

DEFINE VARIABLE is-mandatory                        AS LOGICAL      NO-UNDO.
DEFINE VARIABLE is-mandatory-primary                AS LOGICAL      NO-UNDO.
DEFINE VARIABLE is-mandatory-secondary              AS LOGICAL      NO-UNDO.

    /* Endpoint checking    */

DEFINE VARIABLE is-primary-value-present            AS LOGICAL      NO-UNDO.
DEFINE VARIABLE is-secondary-value-present          AS LOGICAL      NO-UNDO.

/****************************************************************************/

ASSIGN
    is-mandatory                = LOOKUP("mandatory",           cur-flag-list) > 0
    is-mandatory-primary        = LOOKUP("mandatory-primary",   cur-flag-list) > 0  OR is-mandatory
    is-mandatory-secondary      = LOOKUP("mandatory-secondary", cur-flag-list) > 0  OR is-mandatory

    is-primary-value-present    = (cur-primary-value    <> ?)
    is-secondary-value-present  = (cur-secondary-value  <> ?)

&IF DEFINED(ignore-boolean) <> 0 AND DEFINED(ignore-value) <> 0 &THEN           /* HAndle cases where ? is not the only "ignore this?" value    */

    is-primary-value-present    = (is-primary-value-present     AND cur-primary-value     {&ignore-boolean}   {&ignore-value})
    is-secondary-value-present  = (is-secondary-value-present   AND cur-secondary-value   {&ignore-boolean}   {&ignore-value})

&ENDIF
    .

/****************************************************************************/
/* no primary spec'd and this isn't mandatory?      */

IF NOT is-primary-value-present AND
   NOT is-mandatory-primary     THEN
    LEAVE.

    /* If this has no secondary value and secondary's not mandatory,    */
    /* then treat this as a primary value search                        */

IF NOT is-secondary-value-present   AND
   NOT is-mandatory-secondary       THEN

    DO:

    RUN add-qrymgr-query-buffer-search-spec-{&data-type}    (   cur-query-handle,
                                                                   
                                                                cur-buffer-num,

                                                                cur-primary-field-name,

                                                                cur-boolean-op,

                                                                cur-primary-value,

                                                                IF is-mandatory-primary
                                                                    THEN "mandatory"
                                                                    ELSE ""
                                                                ).

    LEAVE.
    END.

    /* Otherwise treat this as a straight compound lookup              */

RUN add-qrymgr-query-buffer-compound-search-spec(   cur-query-handle,
                                                    cur-buffer-num,

                                                    cur-primary-field-name,     cur-secondary-field-name,
                                                    cur-boolean-op,
                                                    STRING(cur-primary-value),  STRING(cur-secondary-value),

                                                    cur-flag-list).

&ENDIF
END PROCEDURE.

