/*----------------------------------------------------------------------------
Program     : utl/qry/qry-mgr-compound.sp

Description : Code to handle compound field range lookups

Copyright(c): Taylor Steel Inc. 2004 $n
Author      : Tim Kuehn
Created     : 3/2/2005 TAYLOR STEEL INC
Notes       :   This assumes the query manager is already in context

PARAMETERS  :
--------Revision History---------------------------------------------------

	Last change: TDK 2006-08-08 08:58:40
----------------------------------------------------------------------------*/

/****************************************************************************/
/* External Files                       */

&IF DEFINED(super-header) = 0 &THEN

{utl/ppmgr/spopen-private-unique-share.i    }                       /* Standard SP header       */

    /* SPs                  */

{utl/qry/qry-mgr.spi    }

    /* Memory management    */

CREATE WIDGET-POOL.

&ENDIF

/****************************************************************************/
/* Standard interfaces for the various data types   */

{utl/qry/qry-mgr-compound.i

    &data-type      = "character"

    &ignore-boolean = ">"
    &ignore-value   = ''

    &*
    }

{utl/qry/qry-mgr-compound.i

    &data-type      = "date"

    &ignore-boolean = "<>"
    &ignore-value   = "?"

    &*
    }
    
{utl/qry/qry-mgr-compound.i

    &data-type      = "decimal"

    &ignore-boolean = "<>"
    &ignore-value   = "0.0"

    &*
    }

{utl/qry/qry-mgr-compound.i

    &data-type      = "integer"

    &ignore-boolean = "<>"
    &ignore-value   = "0"

    &*
    }

/****************************************************************************/
/* Compound ranges  for date-time compound values   */

/******************************************************************************/
PROCEDURE add-qrymgr-query-buffer-compound-search-spec-date-time-range   {&super-header}:
/******************************************************************************
PURPOSE:

    Add a date / time range to a query buffer spec.

PARAMETERS:

    cur-query-handle        Handle to the query

    cur-buffer-num          Buffer position in the query

    cur-date-field-name     Name of the date field to range match against
    cur-time-field-name     Name of the time field to range match against

    cur-start-date          Starting date of date-time range.   Ignored if this is ?
    cur-start-time          Starting time of date-time range.   Ignored if this is ?

    cur-end-date            End date of the date-time range.    Ignored if this is ?
    cur-end-time            End time of the date-time range.    Ignored if this is ?


    cur-flag-list           Optional flags to control behavior.

                            Flag                Description
                            ---------------     ---------------------------------
                            "mandatory"         Don't ignore ? date or time parameter
                                                values

                            "mandatory-date"    Don't ignore ? date parameter values
                                                time values of ? may be ignored if
                                                mandatory-time is not set

                            "mandatory-time"    Don't ignore ? time parameter values
                                                Has no effect if the date is ? and
                                                mandatory / mandatory-date is not set

    NOTE:

        If cur-start-date = cur-end-date AND
           cur-start-time = cur-end-time THEN assume this is an equality match
                                                
******************************************************************************/
/* Local Parameters */

DEFINE INPUT PARAMETER cur-query-handle             AS HANDLE       NO-UNDO.
DEFINE INPUT PARAMETER cur-buffer-num               AS INTEGER      NO-UNDO.

DEFINE INPUT PARAMETER cur-date-field-name          AS CHARACTER    NO-UNDO.
DEFINE INPUT PARAMETER cur-time-field-name          AS CHARACTER    NO-UNDO.

DEFINE INPUT PARAMETER cur-date-start               AS DATE         NO-UNDO.
DEFINE INPUT PARAMETER cur-time-start               AS INTEGER      NO-UNDO.

DEFINE INPUT PARAMETER cur-date-end                 AS DATE         NO-UNDO.
DEFINE INPUT PARAMETER cur-time-end                 AS INTEGER      NO-UNDO.

DEFINE INPUT PARAMETER cur-flag-list                AS CHARACTER    NO-UNDO.

/****************************************************************************/
&IF (DEFINED (super-header) = 0) &THEN
/****************************************************************************/

        /* This is really an equality match     */

IF cur-date-start = cur-date-end    AND
   cur-time-start = cur-time-end    THEN

    DO:

    RUN add-qrymgr-query-buffer-compound-search-spec-date-time( cur-query-handle,
                                                                cur-buffer-num,

                                                                cur-date-field-name,
                                                                cur-time-field-name,

                                                                "=",

                                                                cur-date-start,
                                                                cur-time-start,

                                                                cur-flag-list).

    END.

ELSE    /* This is a range lookup               */

    DO:

    RUN add-qrymgr-query-buffer-compound-search-spec-date-time( cur-query-handle,
                                                                cur-buffer-num,

                                                                cur-date-field-name,
                                                                cur-time-field-name,

                                                                ">=",

                                                                cur-date-start,
                                                                cur-time-start,

                                                                cur-flag-list).


    RUN add-qrymgr-query-buffer-compound-search-spec-date-time( cur-query-handle,
                                                                cur-buffer-num,

                                                                cur-date-field-name,
                                                                cur-time-field-name,

                                                                "<=",

                                                                cur-date-end,
                                                                cur-time-end,

                                                                cur-flag-list).
    END.


&ENDIF
END PROCEDURE.

/******************************************************************************/
PROCEDURE add-qrymgr-query-buffer-compound-search-spec-date-time    {&super-header}:
/******************************************************************************
PURPOSE:


PARAMETERS:

    cur-query-handle            Handle to the query

    cur-buffer-num              Buffer position in the query

    cur-date-field-name         Name of the date field in the buffer
    cur-time-field-name         Name of the time field in the buffer

    cur-boolean-op              One of <>, =, >=, <=, >, <

    cur-primary-value-string    Value of the date field for (range) matching
    cur-secondary-value-string  Value of the time field for (range) matching

    cur-flag-list               List of flags that control behavior

                                Flag                    Description
                                ----------------------  ------------------------------
                                "mandatory"             Both the date and time
                                                        fields are manadatory matches
                                                        even if they're ?

                                "mandatory-primary"     The date match value is a
                                                        manadatory match even if it's
                                                        a ?

                                "mandatory-secondary"   The time match value is a
                                                        manadatory match even if it's
                                                        a ?

	Last change: TSI 3/2/2005 10:56:18 AM
******************************************************************************/
/* Local Parameters */

DEFINE INPUT PARAMETER cur-query-handle             AS HANDLE       NO-UNDO.
DEFINE INPUT PARAMETER cur-buffer-num               AS INTEGER      NO-UNDO.

DEFINE INPUT PARAMETER cur-date-field-name          AS CHARACTER    NO-UNDO.
DEFINE INPUT PARAMETER cur-time-field-name          AS CHARACTER    NO-UNDO.

DEFINE INPUT PARAMETER cur-boolean-op               AS CHARACTER    NO-UNDO.

DEFINE INPUT PARAMETER cur-date-value               AS DATE         NO-UNDO.
DEFINE INPUT PARAMETER cur-time-value               AS INTEGER      NO-UNDO.

DEFINE INPUT PARAMETER cur-flag-list                AS CHARACTER    NO-UNDO.

/****************************************************************************/
&IF (DEFINED (super-header) = 0) &THEN
/****************************************************************************/
/* Local Variables */

DEFINE VARIABLE is-mandatory                        AS LOGICAL      NO-UNDO.
DEFINE VARIABLE is-mandatory-date                   AS LOGICAL      NO-UNDO.
DEFINE VARIABLE is-mandatory-time                   AS LOGICAL      NO-UNDO.

DEFINE VARIABLE cur-date-value-string               AS CHARACTER    NO-UNDO.
DEFINE VARIABLE cur-time-value-string               AS CHARACTER    NO-UNDO.

/****************************************************************************/

ASSIGN
    is-mandatory            = LOOKUP("mandatory",       cur-flag-list) > 0
    is-mandatory-date       = LOOKUP("mandatory-date",  cur-flag-list) > 0  OR is-mandatory
    is-mandatory-time       = LOOKUP("mandatory-time",  cur-flag-list) > 0  OR is-mandatory
    .

/****************************************************************************/
/* no date to spec and this isn't mandatory?    */

IF cur-date-value = ?       AND
   NOT is-mandatory-date    THEN
    LEAVE.

    /* If this has no time value and time's not mandatory,  */
    /* then treat this as a date comparison                 */

IF cur-time-value = ?       AND
   NOT is-mandatory-time    THEN

    DO:

    RUN add-qrymgr-query-buffer-search-spec-date(   cur-query-handle,
                                                                   
                                                    cur-buffer-num,
                                                                   
                                                    cur-date-field-name,
                                                                   
                                                    cur-boolean-op,

                                                    cur-date-value,

                                                    IF is-mandatory-date
                                                        THEN "mandatory"
                                                        ELSE ""
                                                    ).

    LEAVE.
    END.

    /* Do some fiddling before calling the compound query procedure    */

ASSIGN
    cur-date-value-string   = "DATE(" + STRING(INTEGER(cur-date-value)) + ")"
    cur-time-value-string   = STRING(cur-time-value)

    cur-flag-list           = REPLACE(REPLACE(cur-flag-list,    "date",  "primary"),        /* Map the "date" and "time"                */
                                                                "time",  "secondary")       /* flags so they're usable                  */
                                                                                            /* by the compound search spec procedure   */
    .

RUN add-qrymgr-query-buffer-compound-search-spec(   cur-query-handle,

                                                    cur-buffer-num,

                                                    cur-date-field-name,
                                                    cur-time-field-name,

                                                    cur-boolean-op,

                                                    cur-date-value-string,
                                                    cur-time-value-string,

                                                    cur-flag-list).



&ENDIF
END PROCEDURE.

/****************************************************************************/
&IF DEFINED(super-header) = 0 &THEN
/****************************************************************************/
/*                                                                          */
/*                              PRIVATE CODE                                */
/*                                                                          */
/****************************************************************************/

/****************************************************************************/
PROCEDURE construct                                                 PRIVATE:
/****************************************************************************
PURPOSE:


PARAMETERS:

*****************************************************************************/

    /* Required SP's    */

RUN utl/qry/qry-mgr.sp.

END PROCEDURE.

/****************************************************************************/
PROCEDURE destruct                                                  PRIVATE:
/****************************************************************************
PURPOSE:


PARAMETERS:
****************************************************************************/

DELETE WIDGET-POOL.

END PROCEDURE.

&ENDIF
